@@ -70,6 +70,7 @@ struct _GVfsBackendArchive
 
   GFile *		file;
   ArchiveFile *		files;		/* the tree of files */
+  gsize                 size;
 };
 
 G_DEFINE_TYPE (GVfsBackendArchive, g_vfs_backend_archive, G_VFS_TYPE_BACKEND)
@@ -83,6 +84,7 @@ typedef struct {
   GFile *	    file;
   GFileInputStream *stream;
   GVfsJob *	    job;
+  GVfsBackendArchive *backend;
   GError *	    error;
   guchar	    data[4096];
 } GVfsArchive;
@@ -122,10 +124,10 @@ gvfs_archive_read (struct archive *archive,
   return read_bytes;
 }
 
-static off_t
+static int64_t
 gvfs_archive_skip (struct archive *archive,
 		   void           *data,
-		   off_t	    request)
+		   int64_t         request)
 {
   GVfsArchive *d = data;
 
@@ -156,8 +158,9 @@ gvfs_archive_close (struct archive *archive,
   GVfsArchive *d = data;
 
   DEBUG ("CLOSE\n");
-  g_object_unref (d->stream);
-  d->stream = NULL;
+  if (!d->stream)
+    g_vfs_backend_force_unmount (G_VFS_BACKEND (d->backend));
+  g_clear_object (&d->stream);
   return ARCHIVE_OK;
 }
 
@@ -169,10 +172,10 @@ gvfs_archive_set_error_from_errno (GVfsArchive *archive)
   if (gvfs_archive_in_error (archive))
     return;
 
-  g_vfs_job_failed_literal (archive->job,
-                            G_IO_ERROR,
-                            g_io_error_from_errno (archive_errno (archive->archive)),
-                            archive_error_string (archive->archive));
+  g_set_error_literal (&archive->error,
+		       G_IO_ERROR,
+		       g_io_error_from_errno (archive_errno (archive->archive)),
+		       archive_error_string (archive->archive));
 }
 
 static void 
@@ -205,6 +208,7 @@ gvfs_archive_finish (GVfsArchive *archive)
 {
   gvfs_archive_pop_job (archive);
 
+  g_object_unref (archive->backend);
   archive_read_free (archive->archive);
   g_slice_free (GVfsArchive, archive);
 }
@@ -217,6 +221,7 @@ gvfs_archive_new (GVfsBackendArchive *ba, GVfsJob *job)
   
   d = g_slice_new0 (GVfsArchive);
 
+  d->backend = g_object_ref (ba);
   d->file = ba->file;
   gvfs_archive_push_job (d, job);
 
@@ -352,14 +357,48 @@ create_root_file (GVfsBackendArchive *ba)
   g_object_unref (icon);
 }
 
+/* Read archive entry data blocks to determine file size */
+static int64_t
+archive_entry_determine_size (GVfsArchive          *archive,
+			      struct archive_entry *entry)
+{
+  size_t read, size = 0;
+  int result;
+  const void *block;
+  int64_t offset;
+
+  do
+    {
+      result = archive_read_data_block (archive->archive, &block, &read, &offset);
+      if (result >= ARCHIVE_WARN && result <= ARCHIVE_OK)
+	{
+	  if (result < ARCHIVE_OK) {
+	    DEBUG ("archive_read_data_block: result = %d, error = '%s'\n", result, archive_error_string (archive->archive));
+	    archive_set_error (archive->archive, ARCHIVE_OK, "No error");
+	    archive_clear_error (archive->archive);
+	  }
+
+	  size += read;
+	}
+    }
+  while (result != ARCHIVE_FATAL && result != ARCHIVE_EOF);
+
+  if (result == ARCHIVE_FATAL)
+    gvfs_archive_set_error_from_errno (archive);
+
+  return size;
+}
+
 static void
-archive_file_set_info_from_entry (ArchiveFile *	        file, 
+archive_file_set_info_from_entry (GVfsArchive *         archive,
+				  ArchiveFile *         file,
 				  struct archive_entry *entry,
 				  guint64               entry_index)
 {
   GFileInfo *info = g_file_info_new ();
   GFileType type;
   mode_t mode;
+  int64_t size;
   file->info = info;
 
   DEBUG ("setting up %s (%s)\n", archive_entry_pathname (entry), file->name);
@@ -411,8 +450,18 @@ archive_file_set_info_from_entry (ArchiveFile *	        file,
 				   file->name,
 				   type);
 
-  g_file_info_set_size (info,
-			archive_entry_size (entry));
+  if (archive_entry_size_is_set (entry))
+    {
+      size = archive_entry_size (entry);
+    }
+  else
+    {
+      size = archive_entry_determine_size (archive, entry);
+    }
+  g_file_info_set_size (info, size);
+
+  if (file->name[0] == '.')
+    g_file_info_set_is_hidden (info, TRUE);
 
   mode = archive_entry_perm (entry);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ, TRUE);
@@ -499,12 +548,15 @@ create_file_tree (GVfsBackendArchive *ba, GVfsJob *job)
 							  TRUE);
           /* Don't set info for root */
           if (file != ba->files)
-            archive_file_set_info_from_entry (file, entry, entry_index);
+	    {
+	      archive_file_set_info_from_entry (archive, file, entry, entry_index);
+	      ba->size += g_file_info_get_size (file->info);
+            }
 	  archive_read_data_skip (archive->archive);
 	  entry_index++;
 	}
     }
-  while (result != ARCHIVE_FATAL && result != ARCHIVE_EOF);
+  while (result != ARCHIVE_FATAL && result != ARCHIVE_EOF && !gvfs_archive_in_error (archive));
 
   if (result == ARCHIVE_FATAL)
     gvfs_archive_set_error_from_errno (archive);
@@ -814,8 +866,13 @@ try_query_fs_info (GVfsBackend *backend,
                    GFileInfo *info,
                    GFileAttributeMatcher *attribute_matcher)
 {
+  GVfsBackendArchive *ba = G_VFS_BACKEND_ARCHIVE (backend);
+
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, TRUE);
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USE_PREVIEW, G_FILESYSTEM_PREVIEW_TYPE_IF_LOCAL);
+  g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE, ba->size);
+  g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, 0);
+  g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USED, ba->size);
   g_vfs_job_succeeded (G_VFS_JOB (job));
   return TRUE;
 }
